import unittest

try:
    from unittest.mock import Mock
except ImportError:
    from mock import Mock

from fdamod import FDASolver


class TestFDASolver(unittest.TestCase):

    def test_solver(self):

        client = Mock()
        data = {'id': 'fujitsuDA2PT', 'properties': {}}
        solver = FDASolver(client, data)

        # test sample_ising
        linear = {1: -2}
        quadratic = {(1, 2): -1}
        solver.sample_ising(linear, quadratic, encode_type='sparse')
        (body, future), _ = client._submit.call_args
        body['qubo'].sort(key=lambda x: x[1])
        expected_body = {'qubo': [(-2, (1, 1)), (-4, (1, 2)), (2, (2, 2))],
                         'solver_name': 'fujitsuDA2PT',
                         'encode_type': 'sparse',
                         'is_async': True,
                         'parameters': {}}
        self.assertEqual(body, expected_body)

        # test sample_qubo with sparse encoding
        Q = {(1, 2): -2}
        solver.sample_qubo(Q, encode_type='sparse')
        (body, future), _ = client._submit.call_args
        body['qubo'].sort(key=lambda x: x[1])
        expected_body = {'qubo': [(0, (1, 1)), (-2, (1, 2)), (0, (2, 2))],
                         'solver_name': 'fujitsuDA2PT',
                         'encode_type': 'sparse',
                         'is_async': True,
                         'parameters': {}}
        self.assertEqual(body, expected_body)

        # test sample_qubo with dense encoding
        Q = {(0, 1): 0, (1, 2): -2}
        solver.sample_qubo(Q, encode_type='dense')
        (body, future), _ = client._submit.call_args
        expected_body = {'qubo': [[0.0, 0.0, 0.0],
                                  [0.0, 0.0, -2.0],
                                  [0.0, 0.0, 0.0]],
                         'solver_name': 'fujitsuDA2PT',
                         'encode_type': 'dense',
                         'is_async': True,
                         'parameters': {}}
        self.assertEqual(body, expected_body)
